"use strict";

var assert = require('assert');

var AsyncTestUtil = require('async-test-util');

var clone = require('clone');

var pushAtSortPosition = require('../').pushAtSortPosition;

describe('unit.test.js', function () {
  var generateItem = function generateItem() {
    var age = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : AsyncTestUtil.randomNumber();
    return {
      name: AsyncTestUtil.randomString(),
      age: age
    };
  };

  var comparator = function comparator(a, b) {
    if (a.age > b.age) {
      return 1;
    }

    if (a.age === b.age) {
      return 0;
    }

    if (a.age < b.age) {
      return -1;
    }
  };

  var basicArraySize = 40;

  var basicArray = function basicArray() {
    var t = 9;
    return new Array(basicArraySize).fill(0).map(function () {
      t++;
      return generateItem(t);
    });
  };

  describe('sort correctness', function () {
    it('insert at first', function () {
      var before = basicArray();
      var after = pushAtSortPosition(before, generateItem(1), comparator);
      assert.strictEqual(after.length, basicArraySize + 1);
      assert.strictEqual(after[0].age, 1);
    });
    it('insert at last', function () {
      var before = basicArray();
      var after = pushAtSortPosition(before, generateItem(100), comparator);
      assert.strictEqual(after.length, basicArraySize + 1);
      assert.strictEqual(after.pop().age, 100);
    });
    it('should be equal to normal sort', function () {
      var items = new Array(10).fill(0).map(function () {
        return generateItem(AsyncTestUtil.randomNumber(10, 1000));
      });
      var normalSorted = items.sort(comparator);
      console.log('nnnn');
      console.dir(normalSorted);
      var own = [];
      items.forEach(function (item) {
        own = pushAtSortPosition(own, item, comparator);
      });
      assert.deepStrictEqual(normalSorted, own);
    });
  });
  describe('performance', function () {
    it('should use less compare-runs then full sort', function () {
      var c = 0;

      var comparatorCount = function comparatorCount(a, b) {
        c++;

        if (a.age > b.age) {
          return 1;
        }

        if (a.age === b.age) {
          return 0;
        }

        if (a.age < b.age) {
          return 1;
        }
      };

      var before = basicArray();
      var after = pushAtSortPosition(before, generateItem(100), comparatorCount);
      assert.ok(after);
      assert.ok(c < 10);
    });
  });
  describe('other', function () {
    it('should not have mutated the input', function () {
      var before = basicArray();
      var after = pushAtSortPosition(before, generateItem(100), comparator);
      assert.ok(before !== after);
    });
  });
});